#include "bme280.h"
#include "Master.h"
#include "SMBus.h"
#include "math.h"
#include "ToCoNet.h"
#include <utils.h>
#include <string.h>
#include <AppHardwareApi.h>

void bme280_setUp();
void CBme280_getTrim();
double bme280_compensate_T(int32_t adc_T);
double bme280_compensate_P(int32_t adc_P);
unsigned long int bme280_calibration_H();
long  bme280_calibration_T();
unsigned long int bme280_calibration_P();
double bme280_getTemparature();
double bme280_getHumidity();
double bme280_getPressure();

long  adc_data;
int32_t m_fine;
int16_t m_digT1;
int16_t m_digT2;
int16_t m_digT3;
uint16_t m_digP1;
int16_t m_digP2;
int16_t m_digP3;
int16_t m_digP4;
int16_t m_digP5;
int16_t m_digP6;
int16_t m_digP7;
int16_t m_digP8;
int16_t m_digP9;
int8_t m_digH1;
int16_t m_digH2;
int8_t m_digH3;
int16_t m_digH4;
int16_t m_digH5;
int8_t m_digH6;

void bme280_init()
{
    bme280_setUp();
    CBme280_getTrim();
}


void bme280_setUp()
{
    unsigned char osrs_t = 1; //Temperature oversampling 
    unsigned char osrs_p = 1; //Pressure oversampling 
    unsigned char osrs_h = 4; //Humidity oversampling
    unsigned char mode = 3; //Normal mode
    unsigned char t_sb = 5; //Tstandby 1000ms
    unsigned char filter = 3; //Filter off 
    unsigned char spi3w_en = 0; //3-wire SPI Disable

    unsigned char ctrl_meas_reg = (osrs_t << 5) | (osrs_p << 2) | mode;
    unsigned char config_reg = (t_sb << 5) | (filter << 2) | spi3w_en;
    unsigned char ctrl_hum_reg = osrs_h;

   if(bSMBusWrite(BME280_ADDRESS, 0xF2, 1, &ctrl_hum_reg) ==FALSE)
   {
   //     SerialDebugSend("0xF2 NG\r\n", 9);
        return;
   }

   if(bSMBusWrite(BME280_ADDRESS, 0xF4, 1, &ctrl_meas_reg) ==FALSE)
   {
        SerialDebugSend("0xF4 NG\r\n", 9);
        return;
   }
  if(bSMBusWrite(BME280_ADDRESS, 0xF5, 1, &config_reg) ==FALSE)
   {
        SerialDebugSend("0xF5 NG\r\n", 9);
        return;
   }
}


void CBme280_getTrim()
{
    uint8_t data[32];
    memset(data, 0, 32);
 
    if(bSMBusRandomRead(BME280_ADDRESS, 0x88,24, &data[0]) == FALSE)
    {
        return;
    }

    m_digT1 = (data[1] << 8) | data[0];
    m_digT2 = (data[3] << 8) | data[2];
    m_digT3 = (data[5] << 8) | data[4];
    m_digP1 = (data[7] << 8) | data[6];
    m_digP2 = (data[9] << 8) | data[8];
    m_digP3 = (data[11] << 8) | data[10];
    m_digP4 = (data[13] << 8) | data[12];
    m_digP5 = (data[15] << 8) | data[14];
    m_digP6 = (data[17] << 8) | data[16];
    m_digP7 = (data[19] << 8) | data[18];
    m_digP8 = (data[21] << 8) | data[20];
    m_digP9 = (data[23] << 8) | data[22];

    if(bSMBusRandomRead(BME280_ADDRESS, 0xa1,1, &data[24]) == FALSE)
    {
        return;
    }

    m_digH1 = data[24];

    if(bSMBusRandomRead(BME280_ADDRESS, 0xe1, 7, &data[25]) == FALSE)
    {
        return;
    }
    m_digH2 = ((int16_t)data[26] << 8) | data[25];
    m_digH3 = data[27];
    m_digH4 = ((int16_t)data[28] << 4) | (0x0F & data[29]);
    m_digH5 = ((int16_t)data[30] << 4) | ((data[29] >> 4) & 0x0F);
    m_digH6 = data[31];
 
}

double bme280_compensate_T(int32_t adc_T)
{
    double var1, var2, T;
    var1 = (((double) adc_T) / 16384.0 - ((double) m_digT1) / 1024.0) * ((double) m_digT2);
    var2 = ((((double) adc_T) / 131072.0 - ((double) m_digT1) / 8192.0) *
            (((double) adc_T) / 131072.0 - ((double) m_digT1) / 8192.0)) * ((double) m_digT3);
    m_fine = (int32_t) (var1 + var2);
    T = (var1 + var2) / 5120.0;
    return T;
}

double bme280_compensate_P(int32_t adc_P)
{
    double var1, var2, p;
    var1 = ( (double) m_fine / 2.0) - 64000.0;
    var2 = var1 * var1 * ((double) m_digP6) / 32768.0;
    var2 = var2 + var1 * ((double) m_digP5) * 2.0;
    var2 = (var2 / 4.0)+(((double) m_digP4) * 65536.0);
    var1 = (((double) m_digP3) * var1 * var1 / 524288.0 + ((double) m_digP2) * var1) / 524288.0;
    var1 = (1.0 + var1 / 32768.0)*((double) m_digP1);
    if (var1 == 0.0)
    {
        return 0; // avoid exception caused by division by zero
    }
    p = 1048576.0 - (double) adc_P;
    p = (p - (var2 / 4096.0)) * 6250.0 / var1;
    var1 = ((double) m_digP9) * p * p / 2147483648.0;
    var2 = p * ((double) m_digP8) / 32768.0;
    p = p + (var1 + var2 + ((double) m_digP7)) / 16.0;
    return p;
}


unsigned long bme280_calibration_H()
{
    int32_t v_x1;
    
    
    v_x1 = (m_fine - ((signed long int)76800));
    v_x1 = (((((adc_data << 14) -(((int32_t)m_digH4) << 20) - (((int32_t)m_digH5) * v_x1)) + 
              ((int32_t)16384)) >> 15) * (((((((v_x1 * ((int32_t)m_digH6)) >> 10) * 
              (((v_x1 * ((int32_t)m_digH3)) >> 11) + ((int32_t) 32768))) >> 10) + ((int32_t)2097152)) * 
              ((int32_t) m_digH2) + 8192) >> 14));
   v_x1 = (v_x1 - (((((v_x1 >> 15) * (v_x1 >> 15)) >> 7) * ((int32_t)m_digH1)) >> 4));
   v_x1 = (v_x1 < 0 ? 0 : v_x1);
   v_x1 = (v_x1 > 419430400 ? 419430400 : v_x1);
   return (uint32_t)(v_x1 >> 12);   
}

signed long int bme280_calibration_T()
{
    
    signed long int var1, var2, T;
    var1 = ((((adc_data >> 3) - (( long)m_digT1<<1))) * ((long)m_digT2)) >> 11;
//    var1 = (adc_data >> 3)  >> 11;
    var2 = (((((adc_data >> 4) - ((long)m_digT1)) * ((adc_data>>4) - ((long)m_digT1))) >> 12) * ((long)m_digT3)) >> 14;
    
    m_fine = var1 + var2;
    T = (m_fine * 5 + 128) >> 8;
    return T; 
}

double bme280_getTemparature()
{
    long d;
    double t;
    uint8_t data[3];
    if(bSMBusRandomRead(BME280_ADDRESS, 0xfa, 3, &data[0]) == FALSE)
    {
        return -256.0;
    }
    
    adc_data = (data[0] << 12) | (data[1] << 4) | (data[2] >> 4);
//    return compensate_T(adc);
    d = bme280_calibration_T();
    t = (double)d / 100.0;
    return t;
 //   return adc_data;
}


// Returns humidity in %rH as as double. Output value of g46.332h represents 46.332 %rH
double bme280_getHumidity()
{
    double var_H;
    uint8_t data[2];
    uint16_t adc_H;
    
    if(bSMBusRandomRead(BME280_ADDRESS, 0xfd, 2, &data[0]) == FALSE)
    {
        return -256.0;
    }
    adc_H = (data[0] << 8) | data[1];

    var_H = (double)m_fine - 76800.0;
    var_H = (adc_H - (((double)m_digH4) * 64.0 + ((double)m_digH5) / 16384.0 * var_H)) *
    (((double)m_digH2) / 65536.0 * (1.0 + ((double)m_digH6) / 67108864.0 * var_H *
    (1.0 + ((double)m_digH3) / 67108864.0 * var_H)));
    var_H = var_H * (1.0 - ((double)m_digH1) * var_H / 524288.0);
    if (var_H > 100.0)
        var_H = 100.0;
    else if (var_H < 0.0)
        var_H = 0.0;
    return var_H;
}


unsigned long int bme280_calibration_P()
{
    signed long int var1, var2;
    unsigned long int P;
    var1 = (((signed long int)m_fine)>>1) - (signed long int)64000;
    var2 = (((var1>>2) * (var1>>2)) >> 11) * ((signed long int)m_digP6);
    var2 = var2 + ((var1*((signed long int)m_digP5))<<1);
    var2 = (var2>>2)+(((signed long int)m_digP4)<<16);
    var1 = (((m_digP3 * (((var1>>2)*(var1>>2)) >> 13)) >>3) + ((((signed long int)m_digP2) * var1)>>1))>>18;
    var1 = ((((32768+var1))*((signed long int)m_digP1))>>15);
    if (var1 == 0)
    {
        return 0;
    }    
    P = (((unsigned long int)(((signed long int)1048576)-adc_data)-(var2>>12)))*3125;
    if(P<0x80000000)
    {
       P = (P << 1) / ((unsigned long int) var1);   
    }
    else
    {
        P = (P / (unsigned long int)var1) * 2;    
    }
    var1 = (((signed long int)m_digP9) * ((signed long int)(((P>>3) * (P>>3))>>13)))>>12;
    var2 = (((signed long int)(P>>2)) * ((signed long int)m_digP8))>>13;
    P = (unsigned long int)((signed long int)P + ((var1 + var2 + m_digP7) >> 4));
    return P;
}


double bme280_getPressure()
{
    uint8_t data[3];
    
    if(bSMBusRandomRead(BME280_ADDRESS, 0xf7, 3, &data[0]) == FALSE)
    {
        return - 256.0;
    }

    adc_data = (data[0] << 12) | (data[1] << 4) | (data[2] >> 4);
    double p = bme280_calibration_P() / 100.0;
    return p;
}

void bme280_getData(BME280_DATA* pData)
{
    pData->temp = bme280_getTemparature();
    pData->humi = bme280_getHumidity();
    pData->press = bme280_getPressure();
}


/* *****************************************************************************
 End of File
 */
